#include "Vector.h"
#include "CSRMatrix.h"
//   
Vector::Vector() : vectorsize(0), vector(nullptr) {}

//   
Vector::Vector(size_t vectorsize) : vectorsize(vectorsize), vector(new double[vectorsize])
{
	for (int i = 0; i < vectorsize; ++i)
	{
		vector[i] = 0;
	}
}

// 
Vector::Vector(size_t vectorsize, double startingvalue) : vectorsize(vectorsize), vector(new double[vectorsize])// ,        
{
	for (int i = 0; i < vectorsize; ++i)
	{
		vector[i] = startingvalue;
	}
	std::cout << "\n" << "  " << this << "\n";
}

//  
Vector::Vector(const Vector& other) : vectorsize(other.vectorsize), vector(new double[other.vectorsize])
{
	for (int i = 0; i < other.vectorsize; ++i)
	{
		this->vector[i] = other.vector[i];
	}
	std::cout << "\n" << "  ! " << this << "\n";
}

// 
Vector::~Vector()
{
	delete[]vector;
	std::cout << "\n" << " ! " << this << "\n";
}

//  

//   = (   )
Vector& Vector::operator = (const Vector& other)
{
	std::cout << "\n" << "   = " << this << "\n";
	this->vectorsize = other.vectorsize;
	if (this->vector != nullptr)
	{
		delete[] this->vector;
	}
	this->vector = new double[other.vectorsize];
	for (int i = 0; i < other.vectorsize; ++i)
	{
		this->vector[i] = other.vector[i];
	}
	return *this;
}

//   
Vector& Vector::operator ++ ()
{
	for (int i = 0; i < vectorsize; ++i)
	{
		this->vector[i] += 1; //  this->vector[i]++;
	}
	return *this;
}

//   
Vector Vector::operator ++ (int value)
{
	Vector temp(*this); //   
	for (int i = 0; i < vectorsize; ++i)
	{
		this->vector[i] += 1; //  this->vector[i]++;
	}
	return temp;
}

//   
Vector& Vector::operator -- ()
{
	for (int i = 0; i < vectorsize; ++i)
	{
		this->vector[i] -= 1;
	}
	return *this;
}

//   
Vector Vector:: operator --(int value)
{
	Vector temp(*this);
	for (int i = 0; i < vectorsize; ++i)
	{
		this->vector[i] -= 1;
	}
	return temp;
}

//   [] 
double& Vector::operator[](size_t index)
{
	return vector[index];
}
const double& Vector::operator [](size_t index) const
{
	return vector[index];
}

//     
Vector Vector::operator + (const Vector& other)
{
	if (this->vectorsize != other.vectorsize)
	{
		throw OutOfRangeException();
	}
	else
	{
		Vector temp(this->vectorsize, 0);
		for (int i = 0; i < this->vectorsize; ++i)
		{
			temp.vector[i] = this->vector[i] + other.vector[i];
		}
		return temp;
	}
}

//     
Vector Vector::operator - (const Vector& other)
{
	if (this->vectorsize != other.vectorsize)
	{
		throw OutOfRangeException();
	}
	else
	{
		Vector temp(this->vectorsize, 0);
		for (int i = 0; i < this->vectorsize; ++i)
		{
			temp.vector[i] = this->vector[i] - other.vector[i];
		}
		return temp;
	}
}

//   +=  
Vector& Vector::operator += (const Vector& other)
{
	if (this->vectorsize != other.vectorsize)
	{
		throw OutOfRangeException();
	}
	else
	{
		for (int i = 0; i < this->vectorsize; ++i)
		{
			this->vector[i] += other.vector[i];
		}
		return *this;
	}
}

//   -=  
Vector& Vector::operator -= (const Vector& other)
{
	if (this->vectorsize != other.vectorsize)
	{
		throw OutOfRangeException();
	}
	else
	{
		for (int i = 0; i < this->vectorsize; ++i)
		{
			this->vector[i] -= other.vector[i];
		}
		return *this;
	}
}

//   +=  double
Vector& Vector::operator += (double other)
{
	for (int i = 0; i < this->vectorsize; ++i)
	{
		this->vector[i] += other;
	}
	return *this;
}

//   -=  double
Vector& Vector::operator -= (double other)
{
	for (int i = 0; i < this->vectorsize; ++i)
	{
		this->vector[i] -= other;
	}
	return *this;
}

//      
Vector Vector::operator *(double value) const
{
	for (int i = 0; i < this->vectorsize; ++i)
	{
		this->vector[i] *= value;
	}
	return *this;
}

//     
double Vector::operator *(const Vector& other)
{
	double multiplication = 0;
	if (this->vectorsize != other.vectorsize)
	{
		throw OutOfRangeException();
	}
	else
	{
		for (int i = 0; i < this->vectorsize; ++i)
		{
			multiplication += this->vector[i] * other.vector[i];
		}
		return multiplication;
	}
}

//   
Vector ::operator double* () const
{
	return vector;
}

//     
size_t Vector::getsize() const
{
	return vectorsize;
}

//     
double Vector::VectorLength() const
{
	double sum = 0;
	for (int i = 0; i < vectorsize; ++i)
	{
		sum += vector[i] * vector[i];
	}
	return sqrt(sum);
}

//      
Vector operator *(double value, const Vector& vector)
{
	Vector temp(vector);
	for (int i = 0; i < temp.vectorsize; ++i)
	{
		temp.vector[i] = value * temp.vector[i];
	}
	return temp;
}

//     
std::ostream& operator <<(std::ostream& out, const Vector& vector)
{
	for (size_t i = 0; i < vector.vectorsize; ++i)
	{
		out << vector[i] << " ";
	}
	return out;//  
}

//     
std::istream& operator >> (std::istream& in, Vector& vector)
{
	for (size_t i = 0; i < vector.getsize(); ++i)
	{
		in >> vector[i];
	}
	return in;
}

//              
